/**
 * @file 	RLEimage.h
 * @brief	Header file for RLEimage.c - RLE image
 *
 * @date	Thu Mar 29 16:41:15  2001 - Original.
 * @author	Binh Bui (Binh.Bui@sophia.inria.fr).
 */


#ifndef _RLEIMAGE_H_
#define _RLEIMAGE_H_

#include <vector>

// #include "sup/CSmartPointer.h"
#include <tr1/memory>



namespace vsip {

struct image;
struct segment;
struct segment_list;
struct blob;

/** A RLEline is a array of successive segments */
struct RLEline {
  /** Number of segments in this line */
  int nb;

  /** First segment index in this line */
  int fi;
  
  /** Pointer to the first segment */
  std::vector < std::tr1::shared_ptr < segment > >::iterator lh;

  /** Added by Christophe Tornieri.
   *  Now there is a new segment list only used for background updating.
   *  This new list doesn't affect previous processes.
   */
  //segment_list* virtual_segments;
};

/** RLEline_t is a pointer to a RLEline structure */
typedef std::tr1::shared_ptr < struct RLEline > RLEline_t;


//Added by Marcos Zuniga for blob vertical split
/** A RLEregionline is an structure storing segment indexes belonging to an horizontal line of a region */
struct RLEregionline {
	/**Vertical coordinate of this line */
	int y;
	/** Number of segments in this line */
	int nb;
	/** First segment index */
	int fi;
};

/** RLEregionline_t is a pointer to a RLEregionline structure */
typedef std::tr1::shared_ptr < struct RLEregionline > RLEregionline_t;


/** 
 * A RLEimage is a set of RLEline lines.
 */
struct RLEimage {
    /** image width */
    int width;
    
    /** image height */
    int height;
    
    /** Vector of successive RLE lines in ROI */
    std::vector< RLEline_t > data;
};


/** RLEimage_t is a pointer to a RLEimage structure */
typedef std::tr1::shared_ptr < struct RLEimage > RLEimage_t;


/**
 * Allocate the memory for a RLE image.
 * @return pointer to new RLE image.
 */
RLEimage_t RLEimage_new();


/**
 * Free a RLE image.
 * @param RLEimg RLE image to free.
 */
RLEline_t RLEimage_free(RLEimage_t RLEimg);

RLEimage_t RLEimage_initialize(RLEimage_t RLEimg, int width, int height);
		    


/**
 * Get a line from a RLEimage.
 * @param img RLE image.
 * @param line_number Line number need to extract.
 * @return Pointer to the line extracted.
 * @note Not check if line_number is outside image.
 */
RLEline_t RLEimage_get_line(RLEimage_t img, int line_number);



/**
 * Convert a RLE image to a binaire image (type image_t). On binaire image, the 
 * moving segments will have white color. The rest has "black" color.
 * @param RLEimg RLE image to convert.
 * @param biniare output binaire image. If binaire is NULL, this function
 * will allocate the memory for it.
 * @return pointer to binaire image.
 */
image* RLEimage_to_binaire(RLEimage_t 	RLEimg, image* 	binaire);



/**
 * Convert a RLE image to a binary raw image (where each pixel is represented
 * by a char). On binaire image, the moving segments will have white color. 
 * The rest has "black" color.
 * The raw data space (equal to img_width*img_height*sizeof(char) has to be
 * allocated BEFORE calling the function. binary_raw_space points to this space).
 * Warning: only "white" moving data is written, that means that memory space has
 * to be previously reset to 0.
 * @param RLEimg RLE image to convert.
 * @param binary_raw_space pointer (char *) to the preallocated memory space 
 * where the raw image will be written. If binaire is NULL, this function
 * returns after an error message.
 */
void RLEimage_to_binary_raw_gray(RLEimage_t	RLEimg, char* binary_raw_space);



/**
 * Clear a binaire image (type image_t) built from a RLEimage.
 * @param RLEimg RLE image from which binaire image is built.
 * @param biniare biniare image to clear.
 * @note To clear a binaire, we need to clear (set to "black" color) only the 
 * moving segments.
 */
void RLEimage_clear_binaire(RLEimage_t	RLEimg, image* binaire);


/**
 * Convert a RLEimage to a image and save the result to PPM file.
 * @param filepath Target file.
 * @param blobs List of blobs detected, we used them only to draw a bouding box
 * for every blob detected on binaire image.
 * @note Save to  PPM format.
 */                                   
void RLEimage_save_binaire(RLEimage_t RLEimg, char *filepath, blob* blobs);
	
} // end namespace vsip	 
		 
#endif /* _RLEIMAGE_H_ */



